#ifdef PRECOMPILEDHEADERS
	#include "Tactical All.h"
#else
	#include "sgp.h"
	#include "overhead.h"
	#include "Debug Control.h"
    #include "item types.h"
	#include "expat.h"
	#include "XML.h"
#endif

struct
{
	PARSE_STAGE	curElement;

	CHAR8		szCharData[MAX_CHAR_DATA_LENGTH+1];
	struct TrainingItem		curTrainingItem;
	struct TrainingItem *	curArray;
	UINT32			maxArraySize;

	UINT32			currentDepth;
	UINT32			maxReadDepth;
}
typedef trainingItemParseData;

static void XMLCALL
trainingItemStartElementHandle(void *userData, const XML_Char *name, const XML_Char **atts)
{
	trainingItemParseData * pData = (trainingItemParseData *)userData;

	if(pData->currentDepth <= pData->maxReadDepth) //are we reading this element?
	{
		if(strcmp(name, "TRAININGITEMLIST") == 0 && pData->curElement == ELEMENT_NONE)
		{
			pData->curElement = ELEMENT_LIST;

			memset(pData->curArray,0,sizeof(struct TrainingItem)*pData->maxArraySize);

			pData->maxReadDepth++; //we are not skipping this element
		}
		else if(strcmp(name, "TRAININGITEM") == 0 && pData->curElement == ELEMENT_LIST)
		{
			pData->curElement = ELEMENT;

            memset(&pData->curTrainingItem,0,sizeof(struct TrainingItem));

			pData->maxReadDepth++; //we are not skipping this element
		}
		else if(pData->curElement == ELEMENT &&
				(strcmp(name, "uiIndex") == 0 ||
                strcmp(name, "bHealth") == 0 ||
                strcmp(name, "bAgility") == 0 ||
                strcmp(name, "bDexterity") == 0 ||
                strcmp(name, "bStrength") == 0 ||
                strcmp(name, "bWisdom") == 0 ||
                strcmp(name, "bLeadership") == 0 ||
                strcmp(name, "bExplosives") == 0 ||
                strcmp(name, "bMedical") == 0 ||
                strcmp(name, "bMechanical") == 0 ||
                strcmp(name, "bMarksmanship") == 0  ))
		{
			pData->curElement = ELEMENT_PROPERTY;

			pData->maxReadDepth++; //we are not skipping this element
		}

		pData->szCharData[0] = '\0';
	}

	pData->currentDepth++;

}

static void XMLCALL
trainingItemCharacterDataHandle(void *userData, const XML_Char *str, int len)
{
	trainingItemParseData * pData = (trainingItemParseData *)userData;

	if( (pData->currentDepth <= pData->maxReadDepth) &&
		(strlen(pData->szCharData) < MAX_CHAR_DATA_LENGTH)
	){
		strncat(pData->szCharData,str,__min((unsigned int)len,MAX_CHAR_DATA_LENGTH-strlen(pData->szCharData)));
	}
}


static void XMLCALL
trainingItemEndElementHandle(void *userData, const XML_Char *name)
{
	trainingItemParseData * pData = (trainingItemParseData *)userData;

	if(pData->currentDepth <= pData->maxReadDepth) //we're at the end of an element that we've been reading
	{
		if(strcmp(name, "TRAININGITEMLIST") == 0)
		{
			pData->curElement = ELEMENT_NONE;
		}
		else if(strcmp(name, "TRAININGITEM") == 0)
		{
			pData->curElement = ELEMENT_LIST;

			if(pData->curTrainingItem.usIndex < pData->maxArraySize)
			{
				pData->curArray[pData->curTrainingItem.usIndex] = pData->curTrainingItem;
			}
		}
		else if(strcmp(name, "usIndex") == 0)
		{
			pData->curElement = ELEMENT;
			pData->curTrainingItem.usIndex	= (UINT32) atol(pData->szCharData);
		}
        else if(strcmp(name, "bHealth") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bHealth	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bAgility") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bAgility	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bDexterity") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bDexterity	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bStrength") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bStrength	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bWisdom") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bWisdom	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bLeadership") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bLeadership	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bMarksmanship") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bMarksmanship	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bExplosives") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bExplosives	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bMechanical") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bMechanical	= (INT8) atol(pData->szCharData);
		}
        else if(strcmp(name, "bMedical") == 0)
		{
			pData->curElement = ELEMENT;
            pData->curTrainingItem.bMedical	= (INT8) atol(pData->szCharData);
		}


		pData->maxReadDepth--;
	}

	pData->currentDepth--;
}




BOOLEAN ReadInTrainingItemStats(STR fileName)
{
	HWFILE		hFile;
	UINT32		uiBytesRead;
	UINT32		uiFSize;
	CHAR8 *		lpcBuffer;
	XML_Parser	parser = XML_ParserCreate(NULL);

	trainingItemParseData pData;

	DebugMsg(TOPIC_JA2, DBG_LEVEL_3, "Loading TrainingItems.xml" );

	// Open armour file
	hFile = FileOpen( fileName, FILE_ACCESS_READ, FALSE );
	if ( !hFile )
		return( FALSE );

	uiFSize = FileGetSize(hFile);
	lpcBuffer = (CHAR8 *) MemAlloc(uiFSize+1);

	//Read in block
	if ( !FileRead( hFile, lpcBuffer, uiFSize, &uiBytesRead ) )
	{
		MemFree(lpcBuffer);
		return( FALSE );
	}

	lpcBuffer[uiFSize] = 0; //add a null terminator

	FileClose( hFile );


	XML_SetElementHandler(parser, trainingItemStartElementHandle, trainingItemEndElementHandle);
	XML_SetCharacterDataHandler(parser, trainingItemCharacterDataHandle);


	memset(&pData,0,sizeof(pData));
	pData.curArray = TRAININGITEMS;
	pData.maxArraySize = 100;

	XML_SetUserData(parser, &pData);


	if(!XML_Parse(parser, lpcBuffer, uiFSize, TRUE))
	{
		CHAR8 errorBuf[511];

		sprintf(errorBuf, "XML Parser Error in TrainingItems.xml: %s at line %d", XML_ErrorString(XML_GetErrorCode(parser)), XML_GetCurrentLineNumber(parser));
		LiveMessage(errorBuf);

		MemFree(lpcBuffer);
		return FALSE;
	}

	MemFree(lpcBuffer);


	XML_ParserFree(parser);


	return( TRUE );
}
